@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2023-2024 -- leonerd@leonerd.org.uk
 
-package meta 0.005;
+package meta 0.006;
 
 use v5.14;
 use warnings;
@@ -11,6 +11,8 @@ use warnings;
 require XSLoader;
 XSLoader::load( __PACKAGE__, our $VERSION );
 
+use Carp;
+
 # Hackery to make warnings::warnif callable from XS, on perls too old to have
 # warnif_at_level
 $^V ge v5.28 or
@@ -22,6 +24,8 @@ C<meta> - meta-programming API
 
 =head1 SYNOPSIS
 
+=for highlighter language=perl
+
    use v5.14;
    use meta;
 
@@ -53,6 +57,17 @@ versions. Once a workable API shape has been found, it is hoped that this
 module will eventually become dual-life and shipped as part of Perl core, as
 the implementation for PPC 0022. See the link in the L</SEE ALSO> section.
 
+This module attempts to find a balance between accurately representing
+low-level concepts within the current implementation of the Perl interpreter,
+while also providing higher-level abstractions that provide useful behaviour
+for code that uses it. One place this can be seen is the lower-level
+L</list_globs> method, which directly maps to the way that GVs are stored in
+symbol table stashes but requires the user to be aware of the GV-less
+optimisised storage of CVs, as compared to the higher-level L</list_symbols>
+method which provides an abstraction over this complication and presents the
+more useful but less accurate impression of separately named symbols that
+neatly map to their values.
+
 I<Since version 0.003_002> all the entry-point functions and constructors in
 this module will provoke warnings in the C<meta::experimental> category. They
 can be silenced by
@@ -134,6 +149,8 @@ I<Since version 0.005.>
 Returns a list of all the globs in the package that may refer to symbols (i.e.
 not subpackages). They are returned in no particular order.
 
+For a more convenient return value form, see also L</list_symbols>.
+
 =head2 list_subpackage_globs
 
 =head2 list_all_globs
@@ -146,6 +163,8 @@ I<Since version 0.005.>
 Returns a list of all the globs in the package that refer to subpackages, or
 all globs, including subpackages. They are returned in no particular order.
 
+For a more convenient return value form, see also L</list_subpackages>.
+
 =head2 get_symbol
 
    $metasym = $metapkg->get_symbol( $name );
@@ -210,8 +229,48 @@ by using L</try_get_symbol>:
    $metapkg->try_get_symbol( '$variable' ) and
       $metapkg->remove_symbol( '$variable' );
 
+=head2 list_symbols
+
+   %sub_metasyms = $metapkg->list_symbols;
+   %sub_metasyms = $metapkg->list_symbols( sigils => $filter );
+
+I<Since version 0.006.>
+
+Returns an even-sized key/value list containing the symbols within the given
+package instance. Each symbol is returned as a pair, with its sigil-prefixed
+basename first, followed by a metasymbol instance representing it. Since the
+sigil-prefixed names must be unique, it is convenient to assign this list into
+a hash. The symbols are returned in no particular order.
+
+If the optional C<sigils> named parameter is given, it should be a string of
+possible symbol sigils (the characters C<$>, C<@>, C<%> or C<&>). In this
+case, only symbols whose sigil is present in this string will be returned.
+
+=head2 list_subpackages
+
+   %sub_metapkgs = $metapkg->list_subpackages;
+
+I<Since version 0.006.>
+
+Returns an even-sized key/value list containing the immediate sub-packages of
+the given package instance. Each sub-package is returned as a pair, with its
+basename first (minus the "::" suffix), followed by a metapackage instance
+representing it. Since the names of each sub-package must be unique, it is
+convenient to assign this list into a hash. The sub-packages are returned in
+no particular order.
+
 =cut
 
+# Named param handling is a lot easier in pureperl
+sub meta::package::list_symbols
+{
+   my ( $self, %params ) = @_;
+   my $sigils = delete $params{sigils};
+   keys %params and
+      croak "Unrecognised named parameters to meta::package::list_symbols: " . join( ", ", sort keys %params );
+   return meta::package::_list_symbols( $self, $sigils );
+}
+
 =head1 METHODS ON METASYMBOLS
 
 =head2 is_glob, is_scalar, ...
